@@ -1904,12 +1904,7 @@ njs_qjs_clear_timeout(JSContext *ctx, JSValueConst this_val, int argc,
 static JSValue
 njs_qjs_process_getter(JSContext *ctx, JSValueConst this_val)
 {
-    char           **ep;
-    JSAtom         atom;
-    JSValue        obj, val, str, name, env;
-    njs_int_t      ret;
-    njs_uint_t     i;
-    const char     *entry, *value;
+    JSValue         obj;
     njs_console_t  *console;
 
     console = JS_GetRuntimeOpaque(JS_GetRuntime(ctx));
@@ -1918,109 +1913,11 @@ njs_qjs_process_getter(JSContext *ctx, JSValueConst this_val)
         return JS_DupValue(ctx, console->process);
     }
 
-    obj = JS_NewObject(ctx);
+    obj = qjs_process_object(ctx, console->argc, (const char **) console->argv);
     if (JS_IsException(obj)) {
         return JS_EXCEPTION;
     }
 
-    ret = qjs_set_to_string_tag(ctx, obj, "process");
-    if (ret == -1) {
-        JS_FreeValue(ctx, obj);
-        return JS_EXCEPTION;
-    }
-
-    val = JS_NewArray(ctx);
-    if (JS_IsException(val)) {
-        JS_FreeValue(ctx, obj);
-        return JS_EXCEPTION;
-    }
-
-    ret = JS_SetPropertyStr(ctx, obj, "argv", val);
-    if (ret == -1) {
-        JS_FreeValue(ctx, obj);
-        JS_FreeValue(ctx, val);
-        return JS_EXCEPTION;
-    }
-
-    for (i = 0; i < console->argc; i++) {
-        str = JS_NewStringLen(ctx, console->argv[i],
-                              njs_strlen(console->argv[i]));
-        if (JS_IsException(str)) {
-            JS_FreeValue(ctx, obj);
-            return JS_EXCEPTION;
-        }
-
-        ret = JS_DefinePropertyValueUint32(ctx, val, i, str, JS_PROP_C_W_E);
-        if (ret == -1) {
-            JS_FreeValue(ctx, obj);
-            return JS_EXCEPTION;
-        }
-    }
-
-    env = JS_NewObject(ctx);
-    if (JS_IsException(obj)) {
-        JS_FreeValue(ctx, obj);
-        return JS_EXCEPTION;
-    }
-
-    ret = JS_SetPropertyStr(ctx, obj, "env", env);
-    if (ret == -1) {
-        JS_FreeValue(ctx, obj);
-        JS_FreeValue(ctx, env);
-        return JS_EXCEPTION;
-    }
-
-    ep = environ;
-
-    while (*ep != NULL) {
-        entry = *ep++;
-
-        value = (const char *) njs_strchr(entry, '=');
-        if (njs_slow_path(value == NULL)) {
-            continue;
-        }
-
-        str = JS_UNDEFINED;
-        name = JS_NewStringLen(ctx, entry, value - entry);
-        if (JS_IsException(name)) {
-            goto error;
-        }
-
-        str = JS_NewStringLen(ctx, value, njs_strlen(value));
-        if (JS_IsException(str)) {
-            goto error;
-        }
-
-        atom = JS_ValueToAtom(ctx, name);
-        if (atom == JS_ATOM_NULL) {
-            goto error;
-        }
-
-        ret = JS_DefinePropertyValue(ctx, env, atom, str, JS_PROP_C_W_E);
-        JS_FreeAtom(ctx, atom);
-        if (ret == -1) {
-error:
-            JS_FreeValue(ctx, name);
-            JS_FreeValue(ctx, str);
-            JS_FreeValue(ctx, obj);
-            return JS_EXCEPTION;
-        }
-
-        JS_FreeValue(ctx, name);
-    }
-
-    ret = JS_SetPropertyStr(ctx, obj, "pid", JS_NewInt32(ctx, getpid()));
-    if (ret == -1) {
-        JS_FreeValue(ctx, obj);
-        return JS_EXCEPTION;
-    }
-
-    ret = JS_SetPropertyStr(ctx, obj, "ppid", JS_NewInt32(ctx, getppid()));
-    if (ret == -1) {
-        JS_FreeValue(ctx, obj);
-        return JS_EXCEPTION;
-    }
-
     console->process = JS_DupValue(ctx, obj);
 
     return obj;
@@ -2583,6 +2480,8 @@ static const JSCFunctionListEntry njs_qjs_global_proto[] = {
 
 
 static const JSCFunctionListEntry njs_qjs_console_proto[] = {
+    JS_PROP_STRING_DEF("[Symbol.toStringTag]", "Console",
+                       JS_PROP_CONFIGURABLE),
     JS_CFUNC_MAGIC_DEF("error", 0, njs_qjs_console_log, NJS_LOG_ERROR),
     JS_CFUNC_MAGIC_DEF("info", 0, njs_qjs_console_log, NJS_LOG_INFO),
     JS_CFUNC_MAGIC_DEF("log", 0, njs_qjs_console_log, NJS_LOG_INFO),
@@ -2685,7 +2584,7 @@ njs_qjs_module_loader(JSContext *ctx, const char *module_name, void *opaque)
     (void) close(info.fd);
 
     if (njs_slow_path(ret != NJS_OK)) {
-        JS_ThrowInternalError(ctx, "while reading \"%*s\" module",
+        JS_ThrowInternalError(ctx, "while reading \"%.*s\" module",
                               (int) info.file.length, info.file.start);
         return NULL;
     }
@@ -2694,7 +2593,7 @@ njs_qjs_module_loader(JSContext *ctx, const char *module_name, void *opaque)
 
     ret = njs_console_set_cwd(console, &info.file);
     if (njs_slow_path(ret != NJS_OK)) {
-        JS_ThrowInternalError(ctx, "while setting cwd for \"%*s\" module",
+        JS_ThrowInternalError(ctx, "while setting cwd for \"%.*s\" module",
                               (int) info.file.length, info.file.start);
         return NULL;
     }
@@ -2759,13 +2658,6 @@ njs_engine_qjs_init(njs_engine_t *engine, njs_opts_t *opts)
         goto done;
     }
 
-    ret = qjs_set_to_string_tag(ctx, obj, "Console");
-    if (ret == -1) {
-        njs_stderror("qjs_set_to_string_tag() failed\n");
-        ret = NJS_ERROR;
-        goto done;
-    }
-
     JS_SetOpaque(obj, &njs_console);
 
     JS_SetPropertyFunctionList(ctx, obj, njs_qjs_console_proto,
@@ -2929,7 +2821,7 @@ njs_engine_qjs_unhandled_rejection(njs_engine_t *engine)
         return -1;
     }
 
-    JS_ThrowTypeError(ctx, "unhandled promise rejection: %*s", (int) len, str);
+    JS_ThrowTypeError(ctx, "unhandled promise rejection: %.*s", (int) len, str);
     JS_FreeCString(ctx, str);
 
     for (i = 0; i < console->rejected_promises->items; i++) {
@@ -3058,7 +2950,6 @@ njs_qjs_object_completions(njs_engine_t *engine, JSContext *ctx,
 
         for (n = 0; n < length; n++) {
             key.start = (u_char *) JS_AtomToCString(ctx, ptab[n].atom);
-            JS_FreeAtom(ctx, ptab[n].atom);
             if (njs_slow_path(key.start == NULL)) {
                 goto fail;
             }
@@ -3101,7 +2992,7 @@ next:
             JS_FreeCString(ctx, (const char *) key.start);
         }
 
-        js_free_rt(JS_GetRuntime(ctx), ptab);
+        qjs_free_prop_enum(ctx, ptab, length);
 
         prototype = JS_GetPrototype(ctx, object);
         if (JS_IsException(prototype)) {
@@ -3125,7 +3016,7 @@ fail:
     }
 
     if (ptab != NULL) {
-        js_free_rt(JS_GetRuntime(ctx), ptab);
+        qjs_free_prop_enum(ctx, ptab, length);
     }
 
     JS_FreeValue(ctx, object);
