@@ -60,6 +60,9 @@
 #define NGX_QJS_CLASS_ID_STREAM_PERIODIC (NGX_QJS_CLASS_ID_OFFSET + 8)
 #define NGX_QJS_CLASS_ID_STREAM_FLAGS (NGX_QJS_CLASS_ID_OFFSET + 9)
 #define NGX_QJS_CLASS_ID_STREAM_VARS (NGX_QJS_CLASS_ID_OFFSET + 10)
+#define NGX_QJS_CLASS_ID_SHARED (NGX_QJS_CLASS_ID_OFFSET + 11)
+#define NGX_QJS_CLASS_ID_SHARED_DICT (NGX_QJS_CLASS_ID_OFFSET + 12)
+#define NGX_QJS_CLASS_ID_SHARED_DICT_ERROR (NGX_QJS_CLASS_ID_OFFSET + 13)
 
 
 typedef struct ngx_js_loc_conf_s ngx_js_loc_conf_t;
@@ -123,7 +126,6 @@ typedef struct {
     ngx_array_t           *imports;                                           \
     ngx_array_t           *paths;                                             \
                                                                               \
-    njs_vm_t              *preload_vm;                                        \
     ngx_array_t           *preload_objects;                                   \
                                                                               \
     size_t                 buffer_size;                                       \
@@ -338,11 +340,11 @@ ngx_engine_t *ngx_qjs_clone(ngx_js_ctx_t *ctx, ngx_js_loc_conf_t *cf,
     void *external);
 void ngx_engine_qjs_destroy(ngx_engine_t *e, ngx_js_ctx_t *ctx,
     ngx_js_loc_conf_t *conf);
-ngx_int_t ngx_qjs_call(ngx_js_ctx_t *ctx, JSValue function,
-    JSValue *argv, int argc);
+ngx_int_t ngx_qjs_call(JSContext *cx, JSValue function, JSValue *argv,
+    int argc);
 ngx_int_t ngx_qjs_exception(ngx_engine_t *e, ngx_str_t *s);
 ngx_int_t ngx_qjs_integer(JSContext *cx, JSValueConst val, ngx_int_t *n);
-ngx_int_t ngx_qjs_string(ngx_engine_t *e, JSValueConst val, ngx_str_t *str);
+ngx_int_t ngx_qjs_string(JSContext *cx, JSValueConst val, ngx_str_t *str);
 
 #define ngx_qjs_prop(cx, type, start, len)                                   \
     ((type == NGX_JS_STRING) ? qjs_string_create(cx, start, len)             \
@@ -355,7 +357,7 @@ ngx_int_t ngx_qjs_string(ngx_engine_t *e, JSValueConst val, ngx_str_t *str);
 #define ngx_qjs_external_pool(cx, e)                                         \
     ((ngx_external_pool_pt) ngx_qjs_meta(cx, 1))(e)
 #define ngx_qjs_external_resolver(cx, e)                                     \
-    ((ngx_external_resolver_pt) ngx_qjs_meta(vm, 2))(e)
+    ((ngx_external_resolver_pt) ngx_qjs_meta(cx, 2))(e)
 #define ngx_qjs_external_resolver_timeout(cx, e)                             \
     ((ngx_external_timeout_pt) ngx_qjs_meta(cx, 3))(e)
 #define ngx_qjs_external_event_finalize(cx)                                  \
@@ -375,8 +377,11 @@ ngx_int_t ngx_qjs_string(ngx_engine_t *e, JSValueConst val, ngx_str_t *str);
 #define ngx_qjs_external_ctx(cx, e)                                          \
     ((ngx_js_external_ctx_pt) ngx_qjs_meta(cx, 11))(e)
 
+extern qjs_module_t  qjs_webcrypto_module;
+extern qjs_module_t  qjs_xml_module;
 extern qjs_module_t  qjs_zlib_module;
 extern qjs_module_t  ngx_qjs_ngx_module;
+extern qjs_module_t  ngx_qjs_ngx_shared_dict_module;
 
 #endif
 
@@ -389,7 +394,6 @@ void ngx_js_logger(ngx_connection_t *c, ngx_uint_t level,
 char * ngx_js_import(ngx_conf_t *cf, ngx_command_t *cmd, void *conf);
 char * ngx_js_engine(ngx_conf_t *cf, ngx_command_t *cmd, void *conf);
 char * ngx_js_preload_object(ngx_conf_t *cf, ngx_command_t *cmd, void *conf);
-ngx_int_t ngx_js_init_preload_vm(ngx_conf_t *cf, ngx_js_loc_conf_t *conf);
 ngx_int_t ngx_js_merge_vm(ngx_conf_t *cf, ngx_js_loc_conf_t *conf,
     ngx_js_loc_conf_t *prev,
     ngx_int_t (*init_vm)(ngx_conf_t *cf, ngx_js_loc_conf_t *conf));
